/*
 * [Mouse.java]
 *
 * Summary: determines where the mouse is on the screen.
 *
 * Copyright: (c) 2003-2017 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.8+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 2004-04-30 initial
 *  1.1 2005-07-30 ANT build, consistent naming.
 *  1.2 2006-03-06 reformat with IntelliJ, add Javadoc.
 *  1.3 2007-05-25 add pad, icon, clean code to IntelliJ inspector standards.
 *  1.4 2008-09-23 fix problem with MS C++ runtime library.
 *  1.5 2012-12-11 add 64-bit support.
 */
package com.mindprod.mouse;

import java.awt.Point;

import static java.lang.System.*;

/**
 * determines where the mouse is on the screen.
 * <p/>
 * Similar to the functionality in Java 1.5+ MouseInfo.getPointerInfo. It can
 * tell where the mouse is even if it is not on this application. Uses JNI native C++ code that only works on XP,
 * Windows 2000 and Windows 7.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.5 2012-12-11 add 64-bit support.
 * @since 2004-04-30
 */
public final class Mouse
    {
    private static final int FIRST_COPYRIGHT_YEAR = 2003;

    /**
     * undisplayed copyright notice
     *
     * @noinspection UnusedDeclaration
     */
    private static final String EMBEDDED_COPYRIGHT =
            "Copyright: (c) 2003-2017 Roedy Green, Canadian Mind Products, http://mindprod.com";

    /**
     * date this version released
     *
     * @noinspection UnusedDeclaration
     */
    private static final String RELEASE_DATE = "2012-12-11";

    /**
     * embedded version string.
     *
     * @noinspection UnusedDeclaration
     */
    private static final String VERSION_STRING = "1.5";

    static
        {
        // get DLL loaded from somewhere on java.library path.
        String dll = "mouse.32";
        try
            {
            // load nativepcclock.dll on path, set up by Java Web Start
            // arch is x86 or amd64
            if ( System.getProperty( "os.arch" ).equals( "amd64" ) )
                {
                dll = "mouse.64";
                }
            // without .dll suffix
            System.loadLibrary( dll );
            }
        catch ( Exception e )
            {
            out.println( "Unable to load " + dll );
            out.println( e.getMessage() );
            System.exit( 1 );
            }
        // if have troubles change this code to use load(
        // "E:\\com\\mindprod\\mouse\\mouse.32.dll" );
        // or similar.
        }

    /**
     * returns current mouse position x,y bundled into an int with y in the high order bits.
     *
     * @return mouse position bundled xy.
     */
    private static native long jniGetMousePosition();

    /**
     * Get version of the DLL. Useful in debugging to make sure you have the latest DLL installed.
     *
     * @return version number, simple integer. Should be 30.
     */
    public static native int getVersion();

    /**
     * Get position of mouse on the screen. 0,0 is the upper left corner.
     *
     * @return Point containing x,y where the mouse in now measured in screen pixels from the top left corner.
     * @noinspection WeakerAccess
     */
    public static Point getWhereMouseIsOnScreen()
        {
        long xy = jniGetMousePosition();
        // y in high order, MS is little endian.
        int y = ( int ) ( xy >>> 32 );
        int x = ( int ) ( xy & 0xffffffffL );
        return new Point( x, y );
        }

    /**
     * test harness
     *
     * @param args not used
     *
     * @noinspection InfiniteLoopStatement
     */
    public static void main( String[] args )
        {
        out.println( "hit ^C to exit." );
        out.println( "version: " + Mouse.getVersion() );
        out.println( "Every 4 seconds displays where mouse is on screen.  Does not have its own Window." );
        while ( true )
            {
            Point p = Mouse.getWhereMouseIsOnScreen();
            out.println( "x=" + p.x + " y=" + p.y );
            try
                {
                Thread.sleep( 4000 );
                }
            catch ( InterruptedException e )
                {
                // no problem
                }
            }
        }
    }
